 
%{                                          /* -*- C++ -*- */
#include "ValueExpressionDriver.H"
#include <errno.h>
%}

%option noyywrap nounput batch debug 
%option stack
%option prefix="ve"

id      [[:alpha:]_][[:alnum:]_]*
int     [[:digit:]]+

exponent_part              [eE][-+]?[[:digit:]]+
fractional_constant        [-]?(([[:digit:]]*"."[[:digit:]]+)|([[:digit:]]+".")|([[:digit:]]))
float                      ((({fractional_constant}{exponent_part}?)|([[:digit:]]+{exponent_part}))|0)

%{
# define YY_USER_ACTION yylloc->columns (yyleng);
%}
%%

%{
    yylloc->step ();
%}

[ \t]+             yylloc->step ();
[\n]+                yylloc->lines (yyleng); yylloc->step ();

[-+*/(),&^<>!?:.]               return yytext[0];


%{
    typedef ve::ValueExpressionParser::token token;
%}

&&                   return token::TOKEN_AND;
\|\|                 return token::TOKEN_OR;
==                   return token::TOKEN_EQ;
!=                   return token::TOKEN_NEQ;
\<=                   return token::TOKEN_LEQ;
\>=                   return token::TOKEN_GEQ;

pow                   return token::TOKEN_pow;
exp                   return token::TOKEN_exp;
log                   return token::TOKEN_log;
mag                   return token::TOKEN_mag;
sin                   return token::TOKEN_sin;
cos                   return token::TOKEN_cos;
tan                   return token::TOKEN_tan;
min                   return token::TOKEN_min;
max                   return token::TOKEN_max;
sqr                   return token::TOKEN_sqr;
sqrt                  return token::TOKEN_sqrt;

pi                    return token::TOKEN_pi;
pos                   return token::TOKEN_position;
fpos                  return token::TOKEN_fposition;
face                  return token::TOKEN_face;
area                  return token::TOKEN_area;
vol                   return token::TOKEN_volume;
dist                  return token::TOKEN_dist;
rdist                 return token::TOKEN_rdist;
rand                  return token::TOKEN_rand;
randNormal            return token::TOKEN_randNormal;
set                   return token::TOKEN_set;

grad                  return token::TOKEN_grad;
curl                  return token::TOKEN_curl;
magSqrGradGrad        return token::TOKEN_magSqrGradGrad;
snGrad                return token::TOKEN_snGrad;
div                   return token::TOKEN_div;
laplacian             return token::TOKEN_laplacian;

integrate             return token::TOKEN_integrate;
sum                   return token::TOKEN_sum;
interpolate           return token::TOKEN_interpolate;
average               return token::TOKEN_average;
reconstruct           return token::TOKEN_reconstruct;

deltaT                return token::TOKEN_deltaT;
time                  return token::TOKEN_time;

vector                 return token::TOKEN_VECTOR;

true                   return token::TOKEN_TRUE;
false                  return token::TOKEN_FALSE;

{float}                {
                       errno = 0;
                       yylval->val = atof(yytext);
                       return token::TOKEN_NUM;
                     }

[xyz]                return yytext[0];

{id}                 {
    Foam::string *ptr=new Foam::string (yytext);
    if(driver.isVector(*ptr)) {
        yylval->vname = ptr; return token::TOKEN_VID;
    } else if(driver.isScalar(*ptr)) {
        yylval->name = ptr; return token::TOKEN_SID;
    } else if(driver.isFaceScalar(*ptr)) {
        yylval->name = ptr; return token::TOKEN_FSID;
    } else if(driver.isFaceVector(*ptr)) {
        yylval->name = ptr; return token::TOKEN_FVID;
    } else if(driver.isCellSet(*ptr)) {
        yylval->name = ptr; return token::TOKEN_SETID;
    } else {
        driver.error (*yylloc, "field "+*ptr+" not existing or of wrong type");
    }
                     }

.                    driver.error (*yylloc, "invalid character");


%%

void ValueExpressionDriver::scan_begin ()
{
    yy_flex_debug = trace_scanning;
    yy_scan_string(content.c_str());
//    if (!(yyin = fopen (file.c_str (), "r")))
//        error (std::string ("cannot open ") + file);
}

void ValueExpressionDriver::scan_end ()
{
//	    fclose (yyin);
}
